;------------------------------------------------------------------------------
; NCB85 CP/M BIOS
; Roman Borik/RM-Team 2/2012, 2/2013
;------------------------------------------------------------------------------

		cpu	8085undoc

jnk		macro	adr
		jnx5	adr
		endm

jk		macro	adr
		jx5	adr
		endm

;------------------------------------------------------------------------------

CPMVer		equ	22		; CP/M verzia 2.2
BIOSVer		equ	11		; verzia BIOSu 1.1

CCPLength	equ	0800h		; dlzka CCP
BDOSLength	equ	0E00h		; dlzka BDOS
BIOSLengthMax	equ	0A00h		; dlzka BIOS (Max)
CcpBdosSec	equ	(CCPLength+BDOSLength)/128 ; dlzka CCP a BDOS v sektoroch

BaseBIOS	equ	0F600h		; bazova adresa BIOSu
BaseCCP		equ	BaseBIOS-(CCPLength+BDOSLength) ; bazova adresa CCP
BaseBDOS	equ	BaseCCP+CCPLength+6 ; bazova adresa BDOSu

NumDisks	equ	4		; pocet diskovych jednotiek
IOByte		equ	0003h		; IO byte (asi nepodstatne)
CurrentDisk	equ	0004h		; default disk
Buffer		equ	0080h		; default DMA adresa
Retry		equ	5		; pocet pokusov pri diskovej chybe
Rst55Vect	equ	002Ch		; vektor prerusenia RST5.5 (USART)
Rst65Vect	equ	0034h		; vektor prerusenia RST6.5 (TIMER)

BEL		equ	7
CR		equ	0Dh
LF		equ	0Ah
EOF		equ	1Ah
ESC		equ	1Bh

;------------------------------------------------------------------------------
; Vstupno vystupne porty
RIOT_CWR	equ	0
RIOT_STAT	equ	0
RIOT_PA		equ	1
RIOT_PB		equ	2
RIOT_PC		equ	3
RIOT_TIML	equ	4
RIOT_TIMH	equ	5

USART_DATA	equ	8
USART_CWR	equ	9
USART_STAT	equ	9

INTR_OUT	equ	1		; maska bitu urcujuceho volny vystup
INTR_IN		equ	8		; maska bitu urcujuceho byte na vstupe

; strankovanie ROM/RAM - hodnoty pre instrukciu SIM
XROM		equ	40h
XRAM		equ	0C0h

; inicializacia USARTu
USART_RESET	equ	01000000b
USART_CMD1	equ	01001110b
USART_CMD2	equ	00010101b

; vstupna frekvencia citaca 307,2 kHz
TIMER_ESC	equ	480 | 80h	; jeden impulz asi po 2ms
TIMER_ERA	equ	15360 | 0C0h	; opakovane impulzy kazdych 50ms

PIO_CMD		equ	39h		; init PIO 
TIMER_STOP	equ	40h		; zastavenie TIMERu
TIMER_START	equ	0C0h		; spustenie TIMERu

;------------------------------------------------------------------------------
Mon85Entry	equ	010Bh		; povodna adresa skoku v MON85
CheckMemAdr	equ	0FFD0h		; adresa pre rutinu testu strankovania

;------------------------------------------------------------------------------
;******************************************************************************
;------------------------------------------------------------------------------

		org	0

		binclude "mon85-v12-ncb85.bin"

		org	4

		dw	Boot		; adresa Boot kodu

;------------------------------------------------------------------------------
;******************************************************************************
;------------------------------------------------------------------------------

		org	4096

Boot:		di
		lxi	sp,0		; zasobnik na koniec pamati
		call	CheckMem	; je strankovanie dostupne ?
		jnz	Mon85Entry	; nie, skoc rovno do MON85

		; inicializacia USART 8251
		xra	a		; Zabezpec aby sme boli v Command mode
		out	USART_CWR	; 3x 00
		out	USART_CWR
		out	USART_CWR
		mvi	a,USART_RESET	; Reset 8251
		out	USART_CWR
		mvi	a,USART_CMD1	; 8 data, 1 stop, x16
		out	USART_CWR	; RxC a TxC = 307,2 kHz -> 19200 Bd
		mvi	a,USART_CMD2	; RTS, DTR, Enable RX, TX
		out	USART_CWR

		lxi	h,Sign		; zobraz hlasku
		call	Prt0

		mvi	b,3		; cakaj na ESC par sekund
St2:		lxi	d,-1
St3:		call	ConInNW		; zisti stav konzoly
		jnz	St4		; nieco bolo stlacene, skoc dopredu
		dcx	d		; zniz pocitadla a pokracuj,
		jnk	St3		;   ak nie su vynulovane
		dcr	b
		jnz	St2
St4:		cpi	ESC		; bolo stlacene ESC ?
		jz	Mon85Entry	; ano, skoc do MON85

		lxi	h,QBaseBIOS	; prekopiruj BIOS na cielove miesto
		lxi	d,BaseBIOS
		lxi	b,BIOSLength-1
St5:		mov	a,m
		stax	d
		inx	h
		inx	d
		dcx	b
		jnk	St5
		jmp	BaseBIOS

Sign:		db	ESC,"[2J"	; zmazanie obrazovky konzoly
		db	ESC,"[24;1H"	; nastavenie kurzora na spodny riadok
		db	"Press Esc for enter to MON85...",0

;------------------------------------------------------------------------------
; Vstup z konzoly bez cakania.
; I: -
; O: Z=1 - nie je na vstupe znak, alebo to bola 0, A=0
;    Z=0, A=znak zo vstupu
; M: AF
; T: 29 / 47
ConInNW:	in	USART_STAT	; vezmi stav 8251
		ani	00000010b	; je znak na vstupe ?
		rz			; nie, vrat sa
		in	USART_DATA	; ano, precitaj znak zo vstupu
		ora	a
		ret

;------------------------------------------------------------------------------
; Vystup na konzolu.
; I: C=znak posielany na vystup
; O: -
; M: AF
ConOutput:	in	USART_STAT	; vezmi stav 8251
		rrc			; je vystup volny ?
		jnc	ConOutput	; nie, cakaj
		mov	a,c		; znak do A
		out	USART_DATA	; posli na vystup
		ret

;------------------------------------------------------------------------------
; Zobrazi na konzole text ukonceny nulou.
; I: HL=adresa textu konceneho nulou
; O:
; M: HL, AF, C
Prt0:		mov	a,m
		inx	h
		ora	a
		rz
		mov	c,a
		call	ConOutput
		jmp	Prt0

;------------------------------------------------------------------------------
; Zisti, ci je dostupne strankovanie pamati.
; Rutina sa prenasa na koniec pamati, aby sme sami seba neodstrankovali.
; I: -
; O: Z=1 - strankovanie je dostupne
;    Z=0 - strankovanie nie je dostupne
; M: HL, DE, B, AF
CheckMem:	lxi	d,CheckMemR+CheckMemL
		lxi	h,CheckMemAdr+CheckMemL
		mvi	b,CheckMemL
CheckMem2:	dcx	d
		dcx	h
		ldax	d
		mov	m,a
		dcr	b
		jnz	CheckMem2
		pchl

CheckMemR:	mvi	a,XRAM		; odpoj ROM
		sim
		lxi	h,0		; od adresy 0 ocakavame RAM
		mov	a,m		; invertuj byte na adrese 0
		cma
		mov	m,a
		cmp	m		; porovnaj, ci skutocne doslo ku zmene
		cma			;   - ROM vs. RAM
		mov	m,a		; vrat povodnu hodnotu
		mvi	a,XROM		; pripoj naspat ROM
		sim
		ret
CheckMemL	equ	$-CheckMemR

;------------------------------------------------------------------------------
;******************************************************************************
;------------------------------------------------------------------------------

QBaseBIOS:	equ	$

		phase	BaseBIOS

; Vstupne body BIOSu
CBOOTE:		jmp	CBOOT		; Cold boot
WBOOTE:		jmp	WBOOT		; Warm Boot
		jmp	CONST		; Console status
		jmp	CONIN		; Console input
		jmp	CONOUT		; Console output
		jmp	LIST		; List (printer) output
		jmp	PUNCH		; Character output
		jmp	READER		; Character input
		jmp	HOME		; Go to track 0
		jmp	SELDSK		; Select disk
		jmp	SETTRK		; Set track
		jmp	SETSEC		; Set sector
		jmp	SETDMA		; Set DMA
		jmp	READ		; Read sector
		jmp	WRITE		; Write sector
		jmp	LISTST		; List (printer) status
		jmp	SECTRN		; Sector translation

		jmp	ReadByte	; citanie bytu z PMD 32-SD
		jmp	SendCommand	; odoslanie povelu
		jmp	SendByte	; odoslanie bytu
		jmp	SndCrcRdAckErr	; odoslanie CRC a prijatie ACK a ERR
		jmp	ReadAckErr	; prijatie ACK a ERR
		jmp	WaitErrT15	; cakanie na chybovy kod 15s
		jmp	WaitErrT	; cakanie na chybovy kod zvoleny cas
		jmp	SndCrcRdAck	; odoslanie CRC a prijatie ACK
		jmp	ReadCheckCrc	; prijatie a kontrola CRC

;------------------------------------------------------------------------------
; Zahlavie diskovych parametrov
DPH:
DPH0:		dw	0,0		; bez prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adr. buffra, adr. disk. parametrov
		dw	CSV0,ALV0	; adr. zony kontr. suctov a bit. alok. mapy
DPH1:		dw	0,0		; bez prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adr. buffra, adr. disk. parametrov
		dw	CSV1,ALV1	; adr. zony kontr. suctov a bit. alok. mapy
DPH2:		dw	0,0		; bez prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adr. buffra, adr. disk. parametrov
		dw	CSV2,ALV2	; adr. zony kontr. suctov a bit. alok. mapy
DPH3:		dw	0,0		; bez prekladovej tabulky
		dw	0,0
		dw	DIRBUF,DPB	; adresa buffra, adr. disk. parametrov
		dw	CSV3,ALV3	; adr. zony kontr. suctov a bit. alok. mapy

; Tabulka diskovych parametrov:
; disk o velkosti 2MB
; 256 stop, 64 sektorov na stopu
; 1020 alokacnych blokov o velkosti 2kB
; 256 adresarovych poloziek
; 1 systemova stopa
DPB:		dw	64		; SPT - pocet logickych sektorov na stopu
		db	4		; BSH - posun bloku
		db	15		; BLM - maska bloku
		db	0		; EXM - maska rozsirenia
		dw	1019		; DSM - velkost disku-1
		dw	255		; DRM - velkost adresara-1
		db	240		; AL0 - alokacna maska adresara
		db	0		; AL1
		dw	64		; CKS - velkost pola kontrolnych suctov 
		dw	1		; OFF - pocet systemovych stop

;------------------------------------------------------------------------------
Signature:	db	ESC,"[0m"	; reset atributov zobrazenia
		db	ESC,"[2J"	; zmazanie obrazovky konzoly
		db	CR,LF
		db	"NCB85 64k CP/M v"
		db	CPMVer/10+'0','.',CPMVer#10+'0'
		db	", BIOS v"
		db	BIOSVer/10+'0','.',BIOSVer#10+'0'
		db	CR,LF
		db	"Roman Borik/RM-TEAM 02/2013"
TCRLF:		db	CR,LF,0

ErrPMD32:	db	CR,LF
		db	"PMD 32-SD not responding or is not connected!"
		db	CR,LF,BEL,0

ErrBoot:	db	"BOOT Error!"
		db	CR,LF,CR,LF,BEL,0
ErrRead:	db	CR,LF,"Read",0
ErrWrite:	db	CR,LF,"Write",0
ErrError:	db	" Error! - ",0
ErrRetry:	db	" - [R]etry - [I]gnore - Re[B]oot"
		db	CR,LF,BEL,0

		db	0		; 0=teply start, 1=studeny start
CCPCommand:	db	4,"CD *",0
CCPCommandLen	equ	$-CCPCommand

;------------------------------------------------------------------------------
; * CBOOT * Cold boot - studeny start *
CBOOT:		di
		lxi	sp,0		; zasobnik na koniec pamati

		mvi	a,XRAM		; odpoj ROM
		sim

		call	HardwareInit	; inicializuj HW

		lxi	h,Signature
		call	Print0

		xra	a		; vynuluj
		sta	IOByte		;  IOByte
		sta	CurrentDisk	;  aktualny disk a uzivatela
		inr	a
		sta	CCPCommand-1	; flag: studeny start

;------------------------------------------------------------------------------
; * WBOOT * Warm boot - teply start *
WBOOT:		di
		lxi	sp,0		; zasobnik na koniec pamati

		lda	CCPCommand-1
		ora	a
		cz	HardwareInit	; inicializuj HW pri teplom starte
		call	InitSerialInt	; inicializuj 

		mvi	l,5		; 5 pokusov
WBoot1:		call	CheckPMD32	; zisti, ci je pripojena PMD 32-SD
		jnc	WBoot2		; je, skoc dopredu
		call	WaitL
		dcr	l
		jnz	WBoot1

		lxi	h,ErrPMD32	; zobraz chybovu hlasku
		call	Print0
		call	CONIN		; cakaj na klaves
		di			; zakaz prerusenia
		mvi	a,XROM+1Fh	; pripoj ROM a zakaz prerusenia
		sim
		jmp	Mon85Entry	; skoc do MON85

WBoot2:		xra	a		; nastav drive A:, sektor 0 a stopu 0
		sta	Drive
		sta	Sector
		sta	Track
		lxi	h,BaseCCP	; zaciatocna adresa CCP
WBootLoop:	shld	Dma		; priprav DMA
		call	READ		; precitaj sektor
		ora	a
		jnz	WBootErr	; pri chybe skoc
		lda	Sector		; zvys cislo sektora
		inr	a
		sta	Sector
		cpi	CcpBdosSec	; vsetky sektory ?
		jnc	WBootEnd	; ano, skoc ukoncit BOOT
		lhld	Dma		; zvys adresu DMA
		lxi	d,128
		dad	d
		jmp	WBootLoop

WBootErr:	lxi	h,ErrBoot	; zobraz chybovu hlasku
		call	Print0
		call	CONIN		; cakaj na klaves
		jmp	WBOOT		; znovu sa pokus o BOOT

WBootEnd:	lxi	h,Buffer	; predvolena adresa DMA
		shld	Dma
		mvi	a,0C3h		; skok na WBOOT na adresu 0
		sta	0
		lxi	h,WBOOTE
		shld	1
		sta	5		; skok do BDOSu na adresu 5
		lxi	h,BaseBDOS
		shld	6
		lda	CurrentDisk	; naposledy vybrany disk
		mov	c,a		; posli do CCP v registri C
		lxi	h,CCPCommand-1	; studeny/teply start ?
		mov	a,m		; flag do A
		mvi	m,0		; vynuluj flag
		ora	a		; teply start ?
		jz	BaseCCP+3	; skoc do CCP s vyprazdnenim edit buffra
		inx	h		; posun sa na zaciatok "CD *"
		lxi	d,BaseCCP+7	; skopiruj do edit buffra
		mvi	b,CCPCommandLen
		call	CopyBlock
		jmp	BaseCCP		; skok do CCP

;------------------------------------------------------------------------------
CopyBlock:	mov	a,m
		stax	d
		inx	h
		inx	d
		dcr	b
		jnz	CopyBlock
		ret

;------------------------------------------------------------------------------
; * Nepouzite sluzby *
;------------------------------------------------------------------------------
; Test pripravenosti logickeho zariadenia tlace - nepouzite
LISTST:		mvi	a,255		; vrat - vzdy pripraveny, aby sa
					; zabranilo pripadnemu zablokovaniu

;------------------------------------------------------------------------------
; Vystup znaku na logicke zariadenie tlace - nepouzite
LIST:

;------------------------------------------------------------------------------
; Vystup znaku na logicke zariadenie znakoveho vystupu - nepouzite
PUNCH:
		ret

;------------------------------------------------------------------------------
; Vstup znaku z logickeho zariadenia znakoveho vstupu - nepouzite
READER:		mvi	a,EOF		; vrat vzdy znak konca suboru
		ret

;------------------------------------------------------------------------------
; * Konzolove sluzby *
;------------------------------------------------------------------------------
; * Vstup z konzoly *
; - terminaly VT-100, ANSI
; - viac-bytove kody klavesov sa transformuju do jedno-bytovych
;------------------------------------------------------------------------------
; Key        | Scancode | Code |
;------------|----------|------|------------------------------------------------
; Up         | 1B 5B 41 | 0B   | ESC [ A
; Down       | 1B 5B 42 | 0C   | ESC [ B
; Right      | 1B 5B 43 | 0E   | ESC [ C
; Left       | 1B 5B 44 | 0F   | ESC [ D
; Home       | 1B 5B 48 | 06   | ESC [ H - ANSI mode
; End        | 1B 5B 4B | 05   | ESC [ K - ANSI mode
; Num Enter  | 1B 5B 4D | 07   | ESC [ M - VT100 application mode
; F1         | 1B 4F 50 | 01   | ESC O P
; F2         | 1B 4F 51 | 02   | ESC O Q
; F3         | 1B 4F 52 | 03   | ESC O R
; F4         | 1B 4F 53 | 04   | ESC O S
; Num +      | 1B 5B 6C | 1C   | ESC [ l - VT100 application mode
; Num -      | 1B 5B 6D | 1D   | ESC [ m - VT100 application mode
; Num .      | 1B 5B 6E | 1E   | ESC [ n - VT100 application mode
; Num 0      | 1B 5B 70 | 10   | ESC [ p - VT100 application mode
; Num 1      | 1B 5B 71 | 11   | ESC [ q - VT100 application mode
; Num 2      | 1B 5B 72 | 12   | ESC [ r - VT100 application mode
; Num 3      | 1B 5B 73 | 13   | ESC [ s - VT100 application mode
; Num 4      | 1B 5B 74 | 14   | ESC [ t - VT100 application mode
; Num 5      | 1B 5B 75 | 15   | ESC [ u - VT100 application mode
; Num 6      | 1B 5B 76 | 16   | ESC [ v - VT100 application mode
; Num 7      | 1B 5B 77 | 17   | ESC [ w - VT100 application mode
; Num 8      | 1B 5B 78 | 18   | ESC [ x - VT100 application mode
; Num 9      | 1B 5B 79 | 19   | ESC [ y - VT100 application mode
;------------------------------------------------------------------------------
; Backspace  | 08       | 08   | Backspace
; Tab        | 09       | 09   | Tab
; Ctrl+Enter | 0A       | 0A   | Ctrl+Enter / Ctrl+Num Enter
; Enter      | 0D       | 0D   | Enter
; Esc        | 1B       | 1B   | Esc
; Ctrl+BS    | 7F       | 7F   | Delete
;------------------------------------------------------------------------------
; * CONST - Console status - zitenie stavu konzoly *
; Ak bol stlaceny nejaky klaves, jeho kod je ulozeny vo fronte.
; I: -
; O: Z=0, A=255 - bola stlacena klavesa
;    Z=1, A=0   - nebola stlacena klavesa
; M: HL, AF
CONST:
QueueRdAdr:	lxi	h,0		; ukazatel do fronty
		mov	a,m		; je platny znak vo fronte ?
		ora	a
		rz			; nie, vrat sa s A=0
		mvi	a,255		; stlaceny klaves, A=255
		ret

;------------------------------------------------------------------------------
; * CONIN - Console input - cakanie na stlacenie klavesu *
; I: -
; O: A=kod klavesu
; M: HL, BC, AF
CONIN:		lhld	QueueRdAdr+1	; ukazatel do fronty
		mov	a,m		; je platny znak vo fronte ?
		ora	a
		jz	CONIN		; nie, cakaj
		mov	b,a		; uloz kod klavesu do B
		mvi	m,0		; vynuluj aktualnu polozku
		call	IncQueueAdr	; posun ukazatel
		mov	a,b		; kod klavesu nazad do A
		cpi	ESC		; je to ESC ?
		jnz	ConInE		; nie, vrat sa s kodom ESC
		call	ConInWait	; pockaj na dalsi byte z konzoly
		jz	ConInE		; neprisiel, vrat sa s kodom ESC
		sui	'O'		; je to 'O' ?
		jz	ConInA		; ano, skoc dalej
		sui	'['-'O'		; je to '[' ?
		jnz	ConInE		; nie, vrat sa s kodom ESC
ConInA:		mov	m,a		; vynuluj aktualnu polozku
		call	IncQueueAdr	; posun ukazatel
		call	ConInWait	; pockaj na dalsi byte z konzoly
		jz	ConInE		; neprisiel, vrat sa s kodom ESC
		mov	b,a		; uloz si prijaty kod
		mvi	m,0		; vynuluj aktualnu polozku
		call	IncQueueAdr	; posun ukazatel
		mov	a,b
		call	DecodeEsc	; dekoduj Esc kod
		mov	b,a
		jnc	ConInE		; skoc, ak je kod v poriadku
		mvi	b,ESC		; pre neznamy ESC kod vrat samotne ESC
ConInE:		shld	QueueRdAdr+1	; uloz novy ukazatel
		mov	a,b		; kod znaku do A
		ret

;------------------------------------------------------------------------------
; Pocka na byte z konzoly stanoveny cas - asi 2ms.
; I: HL=adresa vo fronte
; O: Z=0, A=byte z konzoly
;    Z=1, A=0, C=0 - neprisiel dalsi byte
; M: B, AF
ConInWait:	mvi	c,212		; pockaj asi 2ms na dalsi byte
ConInWaitL:	mov	a,m		; precitaj byte z fronty
		ora	a		; je tan dalsi znak ?
		rnz			; ano, vrat sa
		dcr	c		; zniz pocitadlo timeoutu
		jnz	ConInWaitL
		ret

;------------------------------------------------------------------------------
; I: A=kod, ktory sa ma prekodovat
; O: CY=0 - A=novy kod
;    CY=1 - chybny vstupny kod
; M: A, C
DecodeEsc:	cpi	'A'		; je to kod >= 'A' ?
		rc			; nie, vrat sa s CY=1
		cpi	'D'+1		; je to kod 'A' az 'D' ?
		jnc	DecodeEsc2	; nie, skoc dalej
		sui	'A'-0Bh		; 'A' -> 0Bh, 'B' -> 0Ch
		cpi	0Dh		; ODh je CR, tak ho vynechame
		cmc
		rnc
		inr	a		; 'C' -> 0Eh, 'D' -> 0Fh
		ora	a
		ret

DecodeEsc2:	cpi	'P'		; je to kod >= 'P' ?
		jc	DecodeEsc3	; nie, skoc dalej
		cpi	'S'+1		; je to kod 'P' az 'S' ?
		jnc	DecodeEsc4	; nie, skoc dalej
		sui	'P'-1		; 'P' az 'S' -> 01h az 04h
		ret

DecodeEsc3:	mvi	c,5		; 'K' -> 05h
		cpi	'K'		; je to kod 'K' ?
		jz	DecodeEsc3A	; ano skoc, dalej
		inr	c		; 'H' -> 06h
		cpi	'H'		; je to kod 'H' ?
		jz	DecodeEsc3A	; ano skoc, dalej
		cpi	'M'		; je to kod 'M' ?
		stc
		rnz			; nie, vrat sa s CY=1
		inr	c		; 'M' -> 07h
DecodeEsc3A:	mov	a,c
		ora	a
		ret

DecodeEsc4:	cpi	'l'		; je to kod >= 'l' ?
		rc			; nie, vrat sa s CY=1
		cpi	'o'		; je to 'o' ?
		cmc
		rz			; ano, vrat sa s CY=1
		jc	DecodeEsc5	; kod >= 'p', skoc
		sui	'l'-1Ch		; 'l' az 'n' -> 1Ch az 1Eh
		ret

DecodeEsc5:	cpi	'y'+1		; je to kod 'p' az 'y' ?
		cmc
		rc			; nie, vrat sa s CY=1
		sui	'p'-10h		; 'p' az 'y' -> 10h az 19h
		ret

;------------------------------------------------------------------------------
; Posun ukazatela do klavesnicovej fronty na dalsiu polozku
; I: HL=adresa do fronty
; O: HL=nova adresa do fronty
; M: AF, HL
IncQueueAdr:	inx	h		; posun ukazatel
		mov	a,l		; dosiahol si koniec fronty ?
		cpi	(KeyQueue+KeyQueSiz) & 255
		rnz			; nie, navrat
		lxi	h,KeyQueue	; nastav sa na zaciatok fronty
		ret

;------------------------------------------------------------------------------
; Inicializacia prerusenia od USARTu.
; I: -
; O: -
; M: AF, HL, B
InitSerialInt:	di			; zakaz prerusenia
		mvi	a,11110b	; povolenie prerusenia RST5.5
		sim			; zapis masku preruseni
		mvi	a,0C3h		; vytvor skokovu instrukciu na rutinu
		sta	Rst55Vect	; pre prijatie bytu z konzoly (USARTu)
		lxi	h,SerialInt
		shld	Rst55Vect+1
		lxi	h,KeyQueue	; inicializuj klavesnicovu frontu
		shld	QueueWrAdr+1	; ukazatel pre zapis do fronty
		shld	QueueRdAdr+1	; ukazatel pre citanie z fronty
		xra	a		; vynuluj frontu
		mvi	b,KeyQueSiz
InitSerialIntL:	mov	m,a		;  aktualny znak vo fronte
		inx	h
		dcr	b
		jnz	InitSerialIntL
		ei			; povol prerusenie
		ret

;------------------------------------------------------------------------------
; Prerusenie od USARTu - prijem znaku z konzoly
; Potrebuje na zasobniku 4 byty - volanie prerusenia a AF.
; HL sa odpamatava osobitne, aby samotne prerusenie nezaberalo v zasobniku
; volajucej strany vela miesta.
; I: -
; O: -
; M: -
SerialInt:	push	psw		; odpamataj AF
		shld	OldHL		; odpamataj HL
		in	USART_DATA	; precitaj znak zo vstupu
		sta	UsartByte
QueueWrAdr:	lxi	h,0		; ukazatel zapisu
		mov	a,m		; je volna polozka ?
		ora	a
		jnz	SerialIntE	; nie, vrat sa z prerusenia
		lda	UsartByte
		mov	m,a		; uloz prijaty kod do fronty
		call	IncQueueAdr	; posun ukazatel
		shld	QueueWrAdr+1	; a uloz
SerialIntE:	lhld	OldHL
		pop	psw
		ei			; povol prerusenie
		ret			; a navrat

;------------------------------------------------------------------------------
; * CONOUT - Console output - Vystup na konzolu *
; I: A alebo C=znak posielany na vystup
; O: -
; M: AF
ConsoleOutputA:	mov	c,a
CONOUT:
ConsoleOutputC:	in	USART_STAT	; vezmi stav 8251
		rrc			; je vystup volny ?
		jnc	ConsoleOutputC	; nie, cakaj
		mov	a,c		; znak do A
		out	USART_DATA	; posli na vystup
		ret

;------------------------------------------------------------------------------
; Zobrazi na konzole text ukonceny nulou.
; I: HL=adresa textu konceneho nulou
; O:
; M: HL, AF, C
Print0:		mov	a,m
		inx	h
		ora	a
		rz
		call	ConsoleOutputA
		jmp	Print0

;------------------------------------------------------------------------------
; Zobrazi chybu citania zapisu v tvare
; Read Error - D/T/S
; Write Error - D/T/S
; D - drive
; T - stopa
; S - sector
PrintError:	call	Print0		; vytlac text Read/Write Error
		lxi	h,ErrError
		call	Print0
		lda	Drive		; zobraz Drive
		adi	'A'
		call	ConsoleOutputA
		mvi	c,'/'
		call	ConsoleOutputC
		lda	Track		; zobraz cislo stopy
		call	Dec8
		mvi	c,'/'
		call	ConsoleOutputC
		lda	Sector		; zobraz cislo sektoru
		call	Dec8
		lxi	h,ErrRetry
		call	Print0
		call	CONIN
		ani	05Fh
		cpi	'B'
		jz	WBOOT
		ret

;------------------------------------------------------------------------------
; Zobrazenie 8 bitoveho cisla na 3 pozicie. Cislo musi byt v rozsahu 0 az 255.
; Prevedie BINarne cislo na BCD a zobrazi na 3 pozicie.
; I: A=cislo - 0 az 255
; O: -
; M: AF, B, DE, HL
Dec8:		mov	h,a		; cislo do H
		mvi	b,8		; 8 bitove cislo
		lxi	d,0		; vynuluj vysledok v DE
Dec8L:		dad	h		; najvyssi bit do CY
		mov	a,e		; jednotky a desiatky
		adc	a		; zdvojnasob a pripocitaj prenos z DAD H
		daa			; preved do BCD
		mov	e,a
		mov	a,d		; stovky
		adc	a		; zdvojnasob a pripocitaj prenos z DAA
		mov	d,a
		dcr	b		; opakuj pre 8 bitov
		jnz	Dec8L
		lxi	h,PrtBCDX+1
		mov	m,b
		mov	a,d
		call	PrtBCDchar
		mov	a,e

;------------------------------------------------------------------------------
; Vytlaci BCD cislo.
; I: A=BCD cislo, HL=PrtBCDX+1
; O: -
; M: AF, B
PrtBCD:		mov	b,a
		rrc
		rrc
		rrc
		rrc
		call	PrtBCDchar
		mov	m,h		
		mov	a,b
PrtBCDchar:	ani	15
		jnz	PrtBCDchar2
PrtBCDX:	mvi	a,0
		ora	a
		rz
		xra	a
PrtBCDchar2:	adi	'0'
		mov	m,a
		push	b
		push	d
		call	ConsoleOutputA
		pop	d
		pop	b
		ret

;------------------------------------------------------------------------------
; * Diskove sluzby *
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
; * SELDSK - Select disk - Vyber disku *
; I: C=cislo disku
; O: HL=adresa tabulky DPH, 0 ak je chybne cislo disku
; M: vsetky
SELDSK:		lxi	h,0		; vrati 0 pri chybe
		mov	a,c
		cpi	NumDisks	; iba disky A: az D:
		rnc			; chyba - HL = 0
		sta	Drive		; drive - uloz do premennej
		mov	l,c		; HL=cislo disku, vypocitaj adresu DPH
		dad	h		; *2
		dad	h		; *4
		dad	h		; *8
		dad	h		; *16
		lxi	d,DPH		; bazova adresa zahlavia disk. param.
		dad	d		; HL=zahlavie parametrov vybraneho disku
		ret

;------------------------------------------------------------------------------
; * HOME - Disk Home - nastavenie hlavy vybranej jednotky na stopu nula *
; I: -
; O: -
; M: A
HOME:		mvi	c,0

;------------------------------------------------------------------------------
; * SETTRK - Set track - nastavenie hlavy vybraneho disku na pozadovanu stopu *
; I: C=cislo stopy
; O: -
; M: A
SETTRK:		mov	a,c
		sta	Track
		ret

;------------------------------------------------------------------------------
; * SETSEC - Set sector - nastavenie cisla sektoru v stope *
; I: BC=cislo sektora (B=0)
; O: -
; M: AF
SETSEC:		mov	a,c		; cislo sektoru 1 az 64
		dcr	a		; uprav rozsah na 0 az 63
		sta	Sector		; uloz novu hodnotu
		ret

;------------------------------------------------------------------------------
; * SETDMA - Set DMA - nastavenie adresy DMA *
; I: BC=adresa DMA
; O: -
; M: HL
SETDMA:		mov	h,b
		mov	l,c
		shld	Dma
		ret

;------------------------------------------------------------------------------
; * SECTRN - Sector translation - preklad logickeho cisla sektoru na fyzicke *
; I: BC=logicke cislo sektora, DE=adresa prekladovej tabulky
; O: HL=cislo fyzickeho sektoru
; M: HL
SECTRN:		mov	h,b		; bez prekladu cisel sektorov
		mov	l,c
		inx	h		; 1 az n
		ret

;------------------------------------------------------------------------------
; * READ - Read sector - citanie sektora *
; I: cislo disku, sektoru a stopy na adresach Drive, Sector, Track
;    zdrojova adresa na adrese Dma
; O: A=0 - OK
;    A=1 - chyba
; M: vsetky
READ:		call	LocalSP		; nastav lokalny zasobnik
READ3:		mvi	b,Retry		; pocet pokusov
READ1:		push	b		; odpamataj
		call	ReadSector	; precitaj sektor
		pop	b		; obnov pocitadlo pokusov
		ana	a		; podarilo sa ?
		jz	READ2		; ano, navrat -> A=0 - OK
		dcr	b		; zniz pocitadlo pokusov
		jnz	READ1		; opakuj
		lxi	h,ErrRead	; text Read Error
		call	PrintError	; zobraz chybu
		cpi	'R'		; Retry ?
		jz	READ3		; ano, skus znovu
		sui	'I'		; Ignore ?
		jz	READ2		; vrat sa A=0
		mvi	a,1		; A=1 - chyba
READ2:		call	RestoreSP	; obnov povodny zasobnik
		ret

;------------------------------------------------------------------------------
; * WRITE - Write sector - zapis sektora *
; I: cislo disku, sektoru a stopy na adresach Drive, Sector, Track
;    zdrojova adresa na adrese Dma
; O: A=0 - OK
;    A=1 - chyba
; M: vsetky
WRITE:		call	LocalSP		; nastav lokalny zasobnik
WRITE3:		mvi	b,Retry		; pocet pokusov
WRITE1:		push	b		; odpamataj
		call	WriteSector	; zapis sektor
		pop	b		; obnov pocitadlo pokusov
		ana	a		; podarilo sa ?
		jz	WRITE2		; ano, navrat -> A=0 - OK
		dcr	b		; zniz pocitadlo pokusov
		jnz	WRITE1		; opakuj
		lxi	h,ErrWrite	; text Write Error
		call	PrintError	; zobraz chybu
		cpi	'R'		; Retry ?
		jz	WRITE3		; ano, skus znovu
		sui	'I'		; Ignore ?
		jz	WRITE2		; vrat sa A=0
		mvi	a,1		; A=1 - chyba
WRITE2:		call	RestoreSP	; obnov povodny zasobnik
		ret

;------------------------------------------------------------------------------
; Nastavi lokalny zasobnik.
; I: -
; O: -
; M: DE, HL, SP
LocalSP:	pop	d		; navratova adresa do DE
		lxi	h,0		; adresa zasobnika do HL
		dad	sp
		lxi	sp,Stack	; nova adresa zasobnika
		push	h		; stara adresa zasobnika na novy zasobnik
		xchg			; nepriamo sa vrat
		pchl

;------------------------------------------------------------------------------
; Nastavi povodny zasobnik.
; I: -
; O: -
; M: DE, HL, SP
RestoreSP:	pop	d		; navratova adresa do DE
		lhld 	Stack-2		; povodny zasobnik do HL
		sphl			; nastav povodnu adresu zasobnika
		xchg			; nepriamo sa vrat
		pchl

;------------------------------------------------------------------------------
; Povel "Q" - precitanie sektora z disku.
; I: cislo disku, sektoru a stopy na adresach Drive, Sector, Track
;    zdrojova adresa na adrese Dma
; O: CY = 1 - chyba timeout, CRC
;    CY = 0 - OK
; M: vsetky
ReadSector:	mvi	a,'Q'		; povel "Q" - precitaj sektor
		call	SendCommand	; odosli na vystup
		rc			; vrat sa, ak doslo ku chybe
		call	SendSecTrk	; posli stopu a sektor
		rc			; vrat sa, ak doslo ku chybe
		call	SndCrcRdAck	; posli CRC a pockaj na potvrdenie
		rc			; vrat sa, ak doslo ku chybe
		mvi	b,0		; vynuluj CRC
		mvi	d,128		; nastav pocitadlo
		lhld	Dma		; do HL adresa pamati
ReadSector2:	call	ReadByte	; precitaj byte
		rc			; vrat sa, ak doslo ku chybe
		mov	m,a		; uloz byte do pamati
		inx	h		; posun ukazatel
		dcr	d		; zniz pocitadlo prijimanych bytov
		jnz	ReadSector2	; ak neboli prijate vsetky, vrat sa do slucky
		jmp	ReadCheckCrc	; skoc precitat CRC

;------------------------------------------------------------------------------
; Povel "T" - zapise sektor na disk.
; I: cislo disku, sektoru a stopy na adresach Drive, Sector, Track
;    zdrojova adresa na adrese Dma
; O: CY = 1 - chyba timeout, CRC
;    CY = 0 - OK
; M: vsetky
WriteSector:	mvi	a,'T'		; povel "T" - zapis sektor
		call	SendCommand	; odosli na vystup
		rc			; vrat sa, ak doslo ku chybe
		call	SendSecTrk	; posli stopu a sektor
		rc			; vrat sa, ak doslo ku chybe
		mvi	d,128		; nastav pocitadlo
		lhld	Dma		; do HL adresa pamati
WriteSector2:	mov	a,m		; byte z pamati
		call	SendByte	; odosli
		rc			; vrat sa, ak doslo ku chybe
		inx	h		; posun ukazatel
		dcr	d		; zniz pocitadlo bytov
		jnz	WriteSector2	; opakuj pre vsetkych 128 bytov
		jmp	SndCrcRdAck	; odosli CRC a cakaj na potvrdenie

;------------------------------------------------------------------------------
; Posle do PMD 32 cislo sektoru vratane cisla disku a cislo stopy.
; I: cislo disku, sektoru a stopy na adresach Drive, Sector, Track
;    B=priebezne CRC
; O: CY = 1 - chyba
;    CY = 0 - OK, B = A = modifikovane CRC
; M: AF, BC
SendSecTrk:	lda	Drive		; cislo disku do 7. a 6. bitu
		ora	a
		jz	SendSecTrk2
       		mvi	c,0C0h
		xri	3
		jz	SendSecTrk3
		rrc
		rrc
SendSecTrk2:	mov	c,a		; C => 00h=A: 80h=B: 40h=C: 0C0h=D:
SendSecTrk3:	lda	Sector		; vezmi sektor
		ora	c		; pridaj disk
		call	SendByte	; a odosli
		rc			; navrat pri chybe
		lda	Track		; vezmi stopu
       		jmp	SendByte	; a odosli

;------------------------------------------------------------------------------
; Posle CRC na vystup a pocka na ACK a vysledok povelu.
; I: B=aktualne CRC
; O: CY=1 - chyba
;    CY=0 - OK, A=vysledok povelu
; M: AF, BC, DE
SndCrcRdAckErr:	mov	a,b		; posielane CRC do A
		call	SendByte	; posli na vystup
		rc			; vrat sa, ak doslo ku chybe
ReadAckErr:	call	ReadByte	; precitaj byte zo vstupu
		rc			; vrat sa, ak doslo ku chybe
		cpi	33h		; bol prijaty byte ACK?
		stc			; nastav chybovy priznak
		rnz			; vrat sa s chybou, ak to nebolo ACK
WaitErrT15:	lxi	d,4500		; timeout asi 15,5 sec
WaitErrT:	push	d		; DE na zasobnik
		mvi	c,INTR_IN
		call	IsIntr		; je platny byte na vstupe?
		pop	d		; obnov DE
		jnc	ReadByte	; ak ano, skoc ho precitat
		dcx	d		; zniz pocitadlo timeoutu
		jnk	WaitErrT	; ak nepretiekol, skoc do slucky
       		stc			; nastav priznak chyby
		ret

;------------------------------------------------------------------------------
; Odosle CRC a caka na potvrdenie. Otestuje chybovy kod na nulu.
; I: B=posielane CRC
; O: CY=1 - chyba
;    CY=0 - OK
; M: AF, BC, DE
SndCrcRdAck:	call	SndCrcRdAckErr	; posli CRC a cakaj ACK a vysledok povelu
		rc			; vrat sa, ak doslo ku chybe
		jmp	ReadCheckCrc2	; ak bol druhy byte skutocne 0, je to OK

;------------------------------------------------------------------------------
; Precita byte zo vstupu, predstavujuci CRC
; I: -
; O: CY=1 - chyba CRC alebo timeout
;    CY=0 - CRC OK
; M: AF, BC
ReadCheckCrc:	call	ReadByte	; precitaj CRC zo vstupu
		rc			; vrat sa, ak doslo ku chybe
		mov	a,b		; skontroluj CRC
ReadCheckCrc2:	ora	a
		rz			; OK
		stc			; nastav priznak chyby
		ret

;------------------------------------------------------------------------------
; Zisti, ci je pripojena PMD 32 a ci odpoveda prezentacnym bytom.
; I: -
; O: CY=1 - chyba, timeout, PMD 32 neodpoveda prezentacnym bytom
; M: AF, DE, BC
CheckPMD32:	lxi     d,20000		; timeout asi 300ms
		mvi	c,INTR_IN
		call    IsIntrD		; cakaj na byte
		rc			; timeout, vrat sa
		in      RIOT_PB		; precitaj prijaty byte
		cpi	0AAh		; je to prezentacny byte 0AAh ?
		stc			; nastav CY ako priznak chyby
		rnz			; nie je to prezent. byte 0AAh, vrat sa s CY
		mvi	a,0AAh		; posli ako odpoved prezent. byte 0AAh
		jmp	SendByte

;------------------------------------------------------------------------------
; Precita byte z PMD 32 a zaroven modifikuje CRC.
; I: B=prieben CRC
; O: CY=1 - chyba
;    CY=0 - A=C=precitany byte, B=modifikovane CRC
; M: AF, BC
ReadByte:	push	d		; DE na zasobnik
		mvi	c,INTR_IN
		call	IsIntr		; je platny byte na vstupe?
		pop	d		; obnov DE
		rc			; nie, vrat sa s chybou
		in	RIOT_PB		; precitaj byte
		mov	c,a		; odloz si ho do C
		xra	b		; modifikuj CRC
		mov	b,a		; a uloz ho naspat do B
		mov	a,c		; vrat nacitany byte do A
		ret

;------------------------------------------------------------------------------
; Posle povel do PMD 32. Pred poslanim povelu sa najprv preveri, ci na zbernici
; nahodou nie je prezentacny byte.
; I: A=povel
; O: CY=1 - chyba
;    CY=0 - OK, B=A=modifikovane CRC
; M: AF, BC
SendCommand:	mov	b,a		; uloz posielany byte do B
		push	b		; BC, DE na zasobnik
		push	d
		call    IsPresent	; zisti, ci je na zbernici prezentacny
					; byte 0AAh, ak ano, odosli naspat
					; prezentacny byte
		pop	d		; obnov DE, BC
		pop	b
		rc			; vrat sa, ak doslo ku chybe
		mov     a,b		; vrat posielany byte do A
		mvi	b,0		; vynuluj CRC
; pokracuj dalej
 
;------------------------------------------------------------------------------
; Posle byte do PMD 32 a modifikuje priebezne CRC.
; I: A=posielany byte, B=priebezne CRC
; O: CY=1 - chyba
;    CY=0 - OK, B=A=modifikovane CRC
; M: AF, BC
SendByte:	push	d		; DE na zasobnik
		mvi	c,INTR_OUT
		call	IsIntr		; zisti, ci je volny vystup
		pop	d		; obnov DE
		rc			; vrat sa, ak doslo ku timeoutu
		out	RIOT_PA		; posli byte na vystup
		xra	b		; modofikuj CRC
		mov	b,a		; a uloz ho naspat do B
		ret

;------------------------------------------------------------------------------
; Zisti, ci na zbernici nie je nahodou prezentacny byte 0AAh, ktory poslala
; disketova jednotka po navrate do "idle" stavu, kedy cakala na povel.
; I: -
; O: CY=1 - chyba
;    CY=0 - OK
; M: AF, C, DE
IsPresent:	call	WaitQ		; male zdrzanie asi 90 us
		in	RIOT_STAT
		ani	INTR_IN		; je platny byte na vstupe?
		rz			; ak nie, vrat sa
IsPresent3:	in	RIOT_PB		; precitaj byte zo vstupu
		cpi	0AAh		; ak je to prezentacny byte 0AAh
		jz	IsPresent2	; skoc poslat prezentacny byte 0AAh na vystup
		lxi	d,7500		; timeout asi 125 ms
		mvi	c,INTR_IN
		call	IsIntrD		; cakaj na byte
		rc			; timeout, vrat sa
		jmp	IsPresent3	; ano, skoc precitat byte zo vstupu
 
IsPresent2:	call	WaitQ		; male zdrzanie asi 90 us
		mvi	a,0AAh		; posli na vystup prezentacny byte 0AAh
		out	RIOT_PA
		lxi     d,7500		; timeout asi 125 ms
		mvi	c,INTR_OUT
		call    IsIntrD		; cakaj na prijatie prezetacneho bytu
		jc	IsPresent3	; ak nebol prijaty, skoc otestovat vstup
		ret			; prezentacny byte bol prijaty,
					; vrat sa s vynulovanym CY

;------------------------------------------------------------------------------
; Zisti, ci je nastavene INTRa alebo INTRb. Na INTR sa caka zvoleny cas.
; INTR=1, ak je platny byte na vstupe, alebo bol precitany byte z vystupu.
; I: C=1 (INTR_OUT) pre port A (vystup)
;    C=8 (INTR_IN) pre port B (vstup)
;    Pri volani od IsIntrD DE=timeout
; O: CY=1 - timeout
;    CY=0 - OK
; M: F, DE
IsIntr:		lxi	d,307		; timeout asi 5ms
IsIntrD:	push	psw		; uloz AF
IsIntr2:	in	RIOT_STAT	; zisti stav INTR
		ana	c
		jnz	IsIntr3		; ak je INTR=1, skoc dopredu
		dcx	d
		jnk	IsIntr2		; a ak nie je nulove, vrat sa do slucky
		pop	psw		; doslo k timeoutu, obnov AF
		stc			; nastav CY ako priznak chyby
		ret
IsIntr3:	pop	psw
		ora	a		; vynuluj CY
		ret

;------------------------------------------------------------------------------
; Kratke zdrzanie
; I: -
; O: kratke zdrzanie asi 90 us
; M: AF
WaitQ:		mvi	a,30
WaitQ1:		dcr	a
		jnz	WaitQ1
		ret

;------------------------------------------------------------------------------
; Dlhe zdrzanie
; I: -
; O: dlhe zdrzanie asi 1s
; M: AF, BC
WaitL:		lxi	b,11100
WaitL1:		call	WaitQ
		dcx	b
		jnk	WaitL1
		ret

;------------------------------------------------------------------------------
; Zdrzanie asi 1ms
;WaitM:		mvi	b,11
;WaitM1:		call	WaitQ
;		dcr	b
;		jnz	WaitM1
;		ret

;------------------------------------------------------------------------------
; Inicializuj USART, PIO a TIMER
; I: -
; O: -
; M: AF
HardwareInit:
; inicializacia USART 8251
		xra	a		; Zabezpec aby sme boli v Command mode
		out	USART_CWR	; 3x 00
		out	USART_CWR
		out	USART_CWR
		mvi	a,USART_RESET	; Reset 8251
		out	USART_CWR
		mvi	a,USART_CMD1	; 8 data, 1 stop, x16
		out	USART_CWR	; RxC a TxC = 307,2 kHz -> 19200 Bd
		mvi	a,USART_CMD2	; RTS, DTR, Enable RX, TX
		out	USART_CWR

		lxi	h,TIMER_ESC
		mvi	c,TIMER_STOP

; inicializacia RIOT 8155
TimerInit:	mov	a,l		; dolnych 8 bitov
		out	RIOT_TIML
		mov	a,h		; hornych 6 bitov + mod
		out	RIOT_TIMH
TimerInitI:	mvi	a,PIO_CMD	; PA riadeny vystup, PB riadeny vstup
		ora	c		; pripoj povel pre TIMER
		out	RIOT_CWR
		ret

;------------------------------------------------------------------------------

BIOSLength	equ	$-BaseBIOS

;------------------------------------------------------------------------------
; Premenne BIOSu
Drive:		db	0		; aktualny disk
Sector:		db	0		; aktualny sektor a disk
Track:		db	0		; aktualna stopa
Dma:		dw	0		; adresa DMA

KeyQueSiz	equ	96
KeyQueue:	ds	KeyQueSiz	; klavsnicovy buffer

StackX:		ds	32		; zasobnik BIOSu
Stack		equ	$

OldHL:		ds	2		; odlozene HL v preruseni
UsartByte	ds	1		; prijaty byte z USARTu

DIRBUF:		ds	128		; buffer pre nacitanie adresara
ALV0:		ds	128		; alokacny vektor disku A:
CSV0:		ds	64		; kontrolne sucty adresara disku A:
ALV1:		ds	128		; alokacny vektor disku B:
CSV1:		ds	64		; kontrolne sucty adresara disku B:
ALV2:		ds	128		; alokacny vektor disku C:
CSV2:		ds	64		; kontrolne sucty adresara disku C:
ALV3:		ds	128		; alokacny vektor disku D:
CSV3:		ds	64		; kontrolne sucty adresara disku D:

;------------------------------------------------------------------------------
XBIOSLength	equ	$-BaseBIOS

		if	XBIOSLength > BIOSLengthMax
		  warning "\aBIOS is too long"
		endif

;------------------------------------------------------------------------------

		dephase

;------------------------------------------------------------------------------

		end

;------------------------------------------------------------------------------
